/*
 *          gAdams - gnu Advantech data acquisition module software
 *
 *   (c) Copyright 2007 Fernando P. Hauscarriaga <fernandoph@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This is the gnu Adam (Advantech data acquisition module) software
 *   for use with the ADAM-4520 RS232/RS485 converter. A set of command
 *   is implemented which can be called to perform different tasks,
 *   for the complete set of command, please read the module's manual.
 *   Note that this software ONLY implements commands for the ADAM-4018M
 *   adquisition module.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
*/
#define _POSIX_SOURCE 1            // We want it portable

#ifndef _GADAMS_H
#define _GADAMS_H

#include <stdio.h>
#include <stdint.h>
#include <limits.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <getopt.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include <my_stdio.h>
#include <parse_tools.h>

#define BUFF 256

#define BAUDRATE B9600

#define CR 0x0D        // Carriage Return

// Input Range Codes

#define	I15MV		0x00  // +/- 15mV
#define I50MV		0x01  // +/- 50mV
#define I100MV		0x02  // +/- 100mV
#define I500MV		0x03  // +/- 500mV
#define I1V		0x04  // +/- 1V
#define I2_5V		0x05  // +/- 2.5V
#define I20MA		0x06  // +/- 20mA
#define THERMOCOUPLE_J  0x0E  // 0 to 760 C
#define THERMOCOUPLE_K  0x0F  // 0 to 1000 C
#define THERMOCOUPLE_T  0x10  // -100 to 400 C
#define THERMOCOUPLE_E  0x01  // 0 to 1000 C
#define THERMOCOUPLE_R  0x12  // 500 to 1750 C
#define THERMOCOUPLE_S  0x13  // 500 to 1750 C
#define THERMOCOUPLE_B  0x14  // 500 to 1800 C
#define PT100           0x09

// Units codes

#define ENG_UNITS	0x00
#define PER_FSR		0x01
#define TWO_COMPL	0x10
#define OHMS		0x11

// Baudrate codes for ADAM data loggers

#define	B12		0x03 	// 1200 bps
#define B24		0x04 	// 2400 bps
#define B48		0x05 	// 4800 bps
#define B96		0x06 	// 9600 bps
#define B192		0x07 	// 19.2 Kbps
#define B384		0x08 	// 38.1 Kbps


/* Structure skeleton for data loggers.
 * 
 * Module --\
 *          |-> File Descriptor (fd).
 *          |-> Device (/dev/something).
 *          |-> Data logger --\
 *          |                 |-> Address (Hex).
 *          |                 |-> Number of Channels used.
 *          |		      |-> Channels (array of int).	
 *          |                 |-> Type of thermocouple used.
 *          |                 \-> * next (pointer to the next data logger).
 *          |
 *          \-> * next (pointer to the next module).
 */

struct logger {
  int address;
  int ch_n;
  int *channel;
  int type;
  struct logger *next;
};

struct module {
  int fd;
  char *device;
  int loggers_n;
  struct logger *loggers;
  struct module *next;
};

/*
 * This are the function prototypes
 */

int make_logger_from_conf (struct logger **logger_conf,
			   struct configuration *conf, int i);
int make_module_from_conf(struct module **module_conf, 
			  struct configuration *conf);
int count_sensors (struct module *mod);
unsigned long get_interval (struct configuration *conf);
int make_log_header (struct module *mod, FILE *common_output);
int print_config (struct module *mod);
int prepare_modules (struct module *mod);
int get_readings (struct module *mod, double *readings);

// Modules' Commands

int module_config  (int fd, int address, int new_address, int type,
		    int baudrate, int moreconfig); // page 4-37
int read_from_channel (int fd, int address, int channel); // page 4-47
int span_calibration (int fd, int address); //page 4-50
int offset_calibration (int fd, int address); // page 4-51
int config_status (int fd, int address); // page 4-41
int en_dis_channels (int fd, int address, int channels_to_manage); // page 4-48
int read_channel_status (int fd, int address); // page 4-49
int sync_sample (int fd); // page 4-52
int read_sync_sample (int fd, int address); // page 4-53
int read_version (int fd, int address); //page 4-43
int read_module_name (int fd, int address); // page 4-44
int CJC_status (int fd, int address); // page 4-56
int CJC_offset_calibration (int fd, int address, char sign); // page 4-57
int set_mem_config (int fd, int address, int channel, int standalone_mode,
		    int data_log_mode, int sample_interval); // page 4-72
int read_mem_config (int fd, int address); // page 4-74
int set_mem_operation_mode (int fd, int address, int mode); // page 4-75
int read_mem_opertaion_mode (int fd, int address); // page 4-76
int event_rec_count (int fd, int address); // page 4-77
int standard_rec_count (int fd, int address); // page 4-78
int read_rec_content (int fd, int address, 
		     int stored_record_index); // page 4-79
int set_alarm_limit (int fd, int address, int channel, int sign_high, 
		     int decimal_high, int high_limit, int sign_low,
     		     int decimal_low, int low_limit); // page 4-81
int read_alarm_limit (int fd, int address, int channel); // page 4-83

// Serial communication modules
int open_modules (struct module **m);
int close_modules (struct module *m);
int sendcmd (int fd, char *cmd);
int read4520 (int fd, char *recv);

#ifdef CHKSUM_ENABLED
int calculate_checksum (char *cmd);
#endif /* CHKSUM_ENABLED */


#endif /* _GADAMS_H */
