/** 
 *   Configuration tools for gAdams
 *
 *   This is an adaptation (shameless copy) of the vac configuration tools from
 *   Federico Bareilles <fede@iar.unlp.edu.ar> for the vacio program.
 *
 *   (c) Copyright 2007 Fernando P. Hauscarriaga <fernandoph@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
  **/

#define _GNU_SOURCE


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <my_stdio.h>
#include <parse_tools.h>

#include "gAdams.h"

#define MAGIC_TAGS "module"
#define MAGIC_TAGS2 "logger"

int print_config (struct module *mod)
{
  struct module *m = mod;
  struct logger *l;
  int i, j, k;

  i = 0;
  while (m->next != NULL){
    printf("Module %d:\n", i);
    printf(" device: %s\n", m->device);
    printf(" Loggers_n: %d\n", m->loggers_n);
    j = 0;
    l = m->loggers;
    while (l->next != NULL){
      printf("   Logger %d:\n", j);
      printf("     Address: 0x%02X\n", l->address);
      printf("     Number of channels: %d\n", l->ch_n);
      for (k = 0; k < m->loggers->ch_n; k++)
	printf("     channel %d", l->channel[k]);
      printf("\n");
      printf("     Type: 0x%02X\n", l->type);
      l = l->next;
      j++;
    }
    m = m->next;
    i++;
  }
  return 0;
}

static struct module *make_module(void)
{
  struct module *m;
  
  if ((m = (struct module *) malloc(sizeof(struct module))) != NULL)
    {
      m->fd = -1;
      m->device = NULL;
      m->loggers_n = -1;
      m->next = NULL;
    }
  return m;
}

static struct logger *make_logger(void)
{
  struct logger *l;
  
  if ((l = (struct logger *) malloc(sizeof(struct logger))) != NULL) {
    l->address = -1;
    l->channel = NULL;
    //l->type = NULL;
    l->next = NULL;
  }
  return l;
}

int make_logger_from_conf (struct logger **logger_conf,
			   struct configuration *conf, int i)
{
  int loggers_n = -1;
  struct configuration *tag;
  int j, k;
  struct logger *l;
  char line[80], c;

  l = *logger_conf = make_logger();

  sprintf(line, "%s.%d.%s.n", MAGIC_TAGS, i, MAGIC_TAGS2);
  if ((tag = find_conf_tag(conf, line)) != NULL){
    loggers_n = atoi(tag->data);
    for (j = 0; j < loggers_n; j++) {
      sprintf (line, "%s.%d.%s.%d.address", MAGIC_TAGS, i, MAGIC_TAGS2, j);
      tag = find_conf_tag (conf, line);
      if (tag != NULL)
	l->address = atoi(tag->data);
      
      sprintf (line, "%s.%d.%s.%d.channels.n", MAGIC_TAGS, i, MAGIC_TAGS2, j);
      tag = find_conf_tag (conf, line);
      if (tag != NULL){
	l->ch_n = atoi(tag->data);
	l->channel = (int *) malloc(l->ch_n);
	for (k = 0; k < l->ch_n; k++){
	  l->channel[k] = -1;
	  sprintf(line, "%s.%d.%s.%d.channels.%d", MAGIC_TAGS, i, 
		  MAGIC_TAGS2, j, k);
	  tag = find_conf_tag (conf, line);
	  if (tag != NULL)
	    l->channel[k] = atoi(tag->data);
	}
      }

      sprintf(line, "%s.%d.%s.%d.type", MAGIC_TAGS, i, MAGIC_TAGS2, j);
      tag = find_conf_tag(conf, line);
      if (tag != NULL) {
	c = tag->data[0];
	switch (c){
	case 'J':
	  l->type = THERMOCOUPLE_J;
	  break;
	case 'K':
	  l->type = THERMOCOUPLE_K;
	  break;
	case 'T':
	  l->type = THERMOCOUPLE_T;
	  break;
	case 'E':
	  l->type = THERMOCOUPLE_E;
	  break;
	case 'R':
	  l->type = THERMOCOUPLE_R;
	  break;
	case 'S':
	  l->type = THERMOCOUPLE_S;
	  break;
	case 'B':
	  l->type = THERMOCOUPLE_B;
	  break;
	case 'P':
	  l->type = PT100;
	  break;	 
	default:
	  printf("Error: Type is not known.");
	  return -1;
	}
      }
          
      l->next = make_logger();
      l = l->next;
    }
  }
  return loggers_n;
}	

int make_module_from_conf(struct module **module_conf, 
			  struct configuration *conf)
{
  int modules_n = -1;
  struct configuration *tag;
  int i, len;
  struct module *m;
  char line[80];
  
  m = *module_conf = make_module();
  if ((tag = find_conf_tag(conf, MAGIC_TAGS".n")) != NULL) {
    modules_n = atoi(tag->data);
    for(i = 0; i < modules_n; i++) {
      sprintf(line, "%s.%d.device",MAGIC_TAGS, i);
      tag = find_conf_tag(conf, line);
      if (tag != NULL) {
	len = strlen(tag->data) + 1;
	m->device = (char *) malloc(len);
	strncpy(m->device, tag->data, len);
      }
      sprintf(line, "%s.%d.%s.n", MAGIC_TAGS, i, MAGIC_TAGS2);

      m->loggers_n = make_logger_from_conf (&m->loggers, conf, i);

      m->next = make_module();
      m = m->next;
    }
  }
  return modules_n;
}

/*
 * count_sensors - Counts all sensors attached.
 */
int count_sensors (struct module *mod)
{
  int total = 0;
  struct logger *l;
  struct module *m;

  // Allocate module structure
  m = mod;

  while (m->next != NULL){
    l = m->loggers;
    while (l->next != NULL){
      total = total + l->ch_n;
      l = l->next;
    }
    m = m->next;
  }
  
  return (total);
}

/*
 * get_interval - Gets interval value from config file
 */

unsigned long get_interval (struct configuration *conf)
{
  struct configuration *tag;
  unsigned long seconds;

  seconds = 0;
  
  if ((tag = find_conf_tag(conf, "interval")) != NULL)
    return (atol(tag->data));
  
  if ((tag = find_conf_tag(conf, "seconds")) != NULL)
    seconds = atol(tag->data);
  seconds = seconds*1000000;
  return (seconds);
  
  return 0;
}

