/*
 *          gAdams - gnu Advantech data acquisition module software
 *
 *   (c) Copyright 2007 Fernando P. Hauscarriaga <fernandoph@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This is the gnu Adam (Advantech data acquisition module) software
 *   for use with the ADAM-4520 RS232/RS485 converter. A set of command
 *   is implemented which can be called to perform different tasks,
 *   for the complete set of command, please read the module's manual.
 *   Note that this software ONLY implements commands for the ADAM-4018M
 *   adquisition module.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
*/

#include <unistd.h>
#include "gAdams.h"

#define PRINT_REG(time_secs, press, instr, common_output) {\
        int __i;\
        fprintf(common_output, "%10.2f", time_secs);\
        for(__i=0;__i<instr;__i++) fprintf(common_output,"\t%5.2f", press[__i]);\
        fprintf(common_output, "\n");\
        fflush(common_output);\
        }

#define DIFTV(t1, t0) (\
(double) (t1.tv_sec - t0.tv_sec) + \
(double) (t1.tv_usec - t0.tv_usec) / 1000000.0)

const char *help_text[] ={
#       ifdef VERBOSE_MODE
  "  -v\t\t - Verbose mode (not implemented yet).",
#       endif
  "  -p\t\t - Print parsed configuration file to stdout.",
  "  -w time\t - Wait until saturation time is apply (sec)",
  "  conf_file\t - Config file. If it is omited, read from stdin.",
  "  -a\t\t - Print data while logging to file (for use with -d).",
  "  -d\t\t - Log readings to a file.",
  "  -c\t\t - Configure modules (remember to short the INIT* terminal)",
  "  -n\t\t - Don't print header to stdout or file.",
  "  -h\t\t - This help !!!.",
  ""
};

int if_dif_temp(double *press0, double *press1, int instr)
{
  int i;
  int ret = 0;
  
  for(i = 0; i < instr; i++) {
    if (press0[i] != press1[i]) {
      ret = 1;
      break;
    }
  }
  
  return ret;
}

int make_log_header (struct module *mod, FILE *common_output)
{
  struct module *m = mod;
  struct logger *l;
  time_t now;
  struct tm date;
  int i, j, k;

  //Timestamp
  time(&now);                     // Takes timestamp in seconds 
  localtime_r(&now, &date);       // Converts seconds into a more-readable date
  date.tm_mon = date.tm_mon++;
  date.tm_year = date.tm_year + 1900;

  fprintf(common_output, "# GADAMS LOG HEADER\n#\n");
  fprintf(common_output, "# DATE: %04d%02d%02d\n", date.tm_year, date.tm_mon,
          date.tm_mday);
  fprintf(common_output, "# TIME: %02d%02d%02d\n#\n", date.tm_hour,
          date.tm_min, date.tm_sec);
  i = 0;
  while (m->next != NULL){
    fprintf(common_output, "# MODULE %d:\n", i);
    fprintf(common_output, "#         device: %s\n", m->device);
    fprintf(common_output, "#         loggers Attached: %d\n", m->loggers_n);
    j = 0;
    l = m->loggers;
    while (l->next != NULL){
      fprintf(common_output, "#         LOGGER %d:\n", j);
      fprintf(common_output, "#                address: 0x%02X\n", l->address);
      fprintf(common_output, "#                Channels to use:");
      for (k = 0; k < l->ch_n; k++)
	fprintf(common_output, " %d", l->channel[k]);
      fprintf(common_output, "\n");
      fprintf(common_output, "#                Type of sensor: 0x%02X\n", 
	      l->type);
      l = l->next;
      j++;
    }
    m = m->next;
    i++;
  }

  return 0;
}

int help(void)
{
  int i=0;
  
  while((char) *help_text[i] != 0) fprintf(stderr,"%s\n",help_text[i++]);
  fprintf(stderr,"\n");
  
  return 0;
}


int main(int argc, char * const argv[])
{
  int op;
  int print_flag = 0;
  int daemon_flag = 0;
  char *filename = NULL;
  char *std_in = "/dev/stdin";
  struct configuration *conf = NULL;
  //struct configuration *tag = NULL;
  //char *tagname = NULL;
  struct module *m = NULL;
  int mod_n;
  int total_sensors;
  unsigned long interval;
  int i /*,j*/;
  double *temp0, *temp1;
  
  time_t timestamp;
  struct tm date;
  FILE *fp = NULL;
  char logfile[18];
  FILE *my_stdout = stdout;
  
  double now_t0;
  int done = 0;
  struct timeval t0, last, now, prev, int_sleep;
  int skip = 0;

  // int tl0, tl1;
  int count;
  int print;
  int stop_flag = 0;
  int print_all_flag = 0;
  int no_header_flag = 0;
  int config_flag = 0;
  time_t wait_until_stop = 0;
  time_t saturation_time = 0;
  
  SET_VERBOSE_FLAG(PERR);
  while  ((op = getopt(argc, argv, "vdanct:pP:c:s:w:")) != EOF)
    switch( op ){
    case 'v':
      INCREMENT_VERBOSE_FLAG();
      break;
    case 'p':
      print_flag = 1;
      break;
    case 'd':
      daemon_flag = 1;
      break;
    case 'c':
      config_flag = 1;
      break;
    case 'a':
      print_all_flag = 1;
      break;
    case 'n':
      no_header_flag = 1;
      break;
    case 's':
      stop_flag = 1;
      saturation_time = (time_t) atol(optarg);
      break;
    case 'w':
      stop_flag = 1;
      wait_until_stop = (time_t) atol(optarg);
      break;
    case 'h':
    default:
      printf("Use %s [-v (incr.)][-p] [conf_file]\n",
	     argv[0]);
      help();
      exit(-1);
    }
  filename = argv[optind];
  if (!filename) filename = std_in;
  if (filename) {
    read_conf_file(&conf, filename);
    interval = get_interval (conf);
    if( expand_conf_braket_array(conf) != 0 ) {
      PVERB(PERR,"%s: bracket not match.\n", filename);
      printf("%s: bracket not match.\n", filename);
    }

    //    if (print_flag)
    //write_conf_file_flag(conf, "/dev/stdout", 0);
    mod_n = make_module_from_conf (&m, conf);

    /* count how many sensors do we have */
    total_sensors = count_sensors(m);
    free_conf(conf); /* or at end :-) */

    if (open_modules(&m) < 0){
      printf("ERROR opening device(s)\n");
      exit (-1);
    }

    if (config_flag == 1){
      if (prepare_modules(m) < 0){
	printf("ERROR prepare_modules()\n");
	exit (-1);
      }
      else {
	printf("Modules configured succesfully.\n");
	exit(0);
      }
    }
	
    if (print_all_flag == 1 && daemon_flag == 0){
      printf ("ERROR: -a implies -d\n");
      exit (-1);
    }

    if (daemon_flag == 1){
      time(&timestamp);
      localtime_r(&timestamp, &date);
      date.tm_mon = date.tm_mon++;
      date.tm_year = date.tm_year + 1900;
      
      sprintf(logfile, "adam-%4d%02d%02d%02d%02d.dat", date.tm_year, 
	      date.tm_mon, date.tm_mday, date.tm_hour, date.tm_min);

      fp = fopen(logfile, "w");

      my_stdout = fp;
      
    }

    if (print_flag){
      print_config(m);
      make_log_header (m, my_stdout);
      exit(0);
    }

    if (no_header_flag == 0)
      make_log_header (m, my_stdout);
    
    /* Allocate memory for store array values type double, a total of
       total_sensors elements per array */

    temp0 = (double *) malloc(sizeof(double) * total_sensors);
    temp1 = (double *) malloc(sizeof(double) * total_sensors);
    interval = 1000000;
    get_readings (m, temp0);
    gettimeofday(&t0, NULL);
    
    memcpy (&last, &t0, sizeof(struct timeval));

    PRINT_REG(0.0, temp0, total_sensors, my_stdout);

    if (print_all_flag == 1) {
      printf("0.0 "); 
      for (i = 0; i < total_sensors; i++)
	printf ("%.2f ", *(temp0+i));
      printf("\n");
    }
    gettimeofday (&now, NULL);
    count = 0;

    while(!done) {
      if(interval >= 100000) {
	unsigned long time;
	gettimeofday(&int_sleep, NULL);
	time = interval - 10000 - (unsigned long)
	  (DIFTV(int_sleep, t0) * 1000000.0 -
	   (double) count * (double)interval);
	//printf("sleeping: %ld usec\n", time);
	if(time < 10000000)
	  usleep(time);
	else
	  usleep(interval - DIFTV(int_sleep, now) * 1000000.0 
	   - 10000);
      }
      gettimeofday(&now, NULL);
      get_readings (m, temp1);

      now_t0 = DIFTV(now, t0);
      print = 0;
      if (if_dif_temp(temp1, temp0, total_sensors)){
	if (skip > 0) {
	  PRINT_REG(DIFTV(prev, t0), temp0, total_sensors, my_stdout);
	  if (print_all_flag == 1) {
	    printf("%.01f ", (double)DIFTV(prev, t0)); 
	    for (i = 0; i < total_sensors; i++)
	      printf ("%.2f ", *(temp0+i));
	    printf("\n");
	  }
	}
	//printf("skip: %d ", skip);
	memcpy(&last, &now, sizeof(struct timeval));
	//memcpy(temp0, temp1, sizeof(double) * instr);
	memcpy(temp0, temp1, sizeof(double) * total_sensors);
	//PRINT_REG(now_t0, temp1, instr, my_stdout);
	PRINT_REG(now_t0, temp1, total_sensors, my_stdout);
	if (print_all_flag == 1) {
	  printf("%.01f ", (double)now_t0); 
	  for (i = 0; i < total_sensors; i++)
	    printf ("%.2f ", *(temp0+i));
	  printf("\n");
	}
	print = 1;
	skip = 0;
      }
      else {
	memcpy(&prev, &now, sizeof(struct timeval));
	skip++;
      }
      
      if (stop_flag &&
	  (time_t) now_t0 >= wait_until_stop &&
	  (time_t) DIFTV(now, last) >= saturation_time   ) {
	done = 1;
      }
      count++;
    }
    
    free(temp0);
    free(temp1);
    fclose(my_stdout);
  }
  return 0;
}
