/*
 *          gAdams - gnu Advantech data acquisition module software
 *
 *   (c) Copyright 2007 Fernando P. Hauscarriaga <fernandoph@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This are the command and communication routines for use with the modules.
 *
 *   Syntax for commands:
 *     [delimiter character][address][command][data][checksum][carriage return]
 *
 *   Commands implemented:
 *   %AANNTTCCFF        ==> Configuration
 *   #AAN               ==> Read Analog Input from Channel N
 *   $AA0               ==> Span Calibration
 *   $AA1               ==> Offset Calibration
 *   $AA5VV             ==> Enable/disable Channels for Multiplexing
 *   $AA6               ==> Read Channel Statuscvs
 *   #**                ==> Syncronized Sampling
 *   $AA4               ==> Read Syncronized Data
 *   $AAF               ==> Read Version
 *   $AAM               ==> Read Module Name
 *   $AA3               ==> CJC Status
 *   $AA9               ==> CJC Offset Calibration
 *   @AACCCSDTTTT       ==> Set Memory Configuration
 *   @AAD               ==> Read Memory Configuration
 *   @AASO              ==> Set Memory Operation Mode
 *   @AAT               ==> Read Memory Operation Mode
 *   @AAL               ==> Event Record Count
 *   @AAN               ==> Standard Record Count
 *   @AARNNNN           ==> Read Record Content
 *   @AAACSDHHHHTEIIII  ==> Set Alarm limit
 *   @AABC              ==> Read Alarm limit
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
*/

// TODO: Configurar dinamicamente las escalas de lectura (ingenieria, Ca2..etc)


#include <unistd.h>
#include "gAdams.h"

int open_modules (struct module **mod)
{
  struct module *m = *mod;
#ifndef TEST_MODE
  struct termios newtio;
#endif

  while (m->next != NULL){
#ifdef TEST_MODE
    m->fd = 3;
#else
    m->fd = open(m->device, O_RDWR);
    if (m->fd < 0)
      return (-1);
    else
      {
	memset(&newtio, 0, sizeof(newtio));
	newtio.c_cflag = (BAUDRATE|CS8|CREAD|CLOCAL);
	newtio.c_iflag &= IXON;
	newtio.c_cc[VMIN]  = 1;
	newtio.c_cc[VTIME] = 0;
	//Set it!
	tcsetattr(m->fd, TCSANOW, &newtio);

	return 1;
      }
#endif /* TEST_MODE */
    m = m->next;
  }
  return 0;
}

int close_modules (struct module *mod)
{
  struct module *m = mod;
  
  while (m->next != NULL){
    close(m->fd);
    m = m->next;
  }
  return 0;
}

int sendcmd (int fd, char *cmd)
{
#ifdef CHKSUM_ENABLED
  int chksum;
#endif
  int snd;
  char *tosend;

#ifdef TEST_MODE
  printf("Command to send: %s\n", cmd);
#endif

#ifdef CHKSUM_ENABLED
  tosend = malloc (strlen(cmd)+3);
  chksum = calculate_checksum(cmd);
  sprintf (tosend, "%s%02X%c", cmd, chksum, CR);
#else
  tosend = malloc (strlen(cmd)+1);
  sprintf (tosend, "%s%c", cmd, CR);
#endif /* CHKSUM_ENABLED */

#ifdef TEST_MODE
  printf("Sended Command: %s\n", tosend);
  snd = strlen(tosend);
#else
  snd = write(fd, tosend, strlen(tosend));
#endif

  free(tosend);

  return snd;
}

#ifdef CHKSUM_ENABLED
int calculate_checksum (char *cmd)
{
  int i, sum;

  sum = 0;

  for (i = 0; i < strlen(cmd); i++){
    sum += *(cmd+i);
    //sum = sum % 0x100;
  }
  if (sum > 0x100)
    sum = sum % 0x100;
  
#ifdef TEST_MODE
  printf("Calculated checksum: 0x%02X\n", sum);
#endif

  return sum;
}    
#endif /* CHKSUM_ENABLED */

int read4520 (int fd, char *recv)
{
  int len = 0;
  int l;
    
  do{
    l = read (fd, recv+len, 1);
    if(*(recv+len) != CR)
      len += l;
  } while (len == 0 || recv[len] != CR);
  
  return (len);
}

/*
 * prepare_modules - Set initial modules/loggers configuration.
 */
int prepare_modules (struct module *mod)
{
  struct module *m = mod;
  struct logger *l;
  int i, fd, config, len;
  char response[BUFF];
 
  config = ENG_UNITS;

#ifdef LINE
  config |= 0x80;
#endif
#ifdef CHKSUM_ENABLED
  config |= 0x40;
#endif
  printf("0x%02X\n", config);
  while (m->next != NULL){
    l = m->loggers;
    fd = m->fd;
    while (l->next != NULL){
      module_config(fd, 0x00, l->address, l->type, B96, config);
      len = read4520(fd, response);
      for (i = 0; i < len; i++)
	printf("%c", *(response+i));
      printf("\n");
      sleep (7);
      //      if ((strcmp(response, "!%02X%c", l->address))<0)
      read4520(fd, response);
      //sleep(7);
      l = l->next;
    }
    m = m->next;
  }
  return 0;
}

int get_readings (struct module *mod, double *readings)
{
  int i, j, fd, len;
  struct module *m = mod;
  struct logger *l;
  char response[BUFF];
  char *readings_char;
  
  while (m->next != NULL){
    l = m->loggers;
    fd = m->fd;
    while (l->next != NULL){
      for (i = 0; i < l->ch_n; i++){
	read_from_channel (fd, l->address, *(l->channel+i));
	len = read4520(fd, response);
	readings_char = malloc (len);
	for (j = 0; j < len; j++)
	  *(readings_char+j) = *(response+j+1);
	*(readings+i) = atof(readings_char);
	free(readings_char);
	usleep(20000);
      }
      l = l->next;
    }
    m = m->next;
  }
  return 0;
}

/*
 * END of I/O comm.
 */

/* ========================================================================= */
/*                       DATA LOGGERS COMMANDS                               */
/* ========================================================================= */
/*
 * module_config - Configure a module
 */
 
int module_config (int fd, int address, int new_address, int type, 
		   int baudrate, int moreconfig)
{
  char cmd[12];
  int snd;

  sprintf(cmd, "%%%02X%02X%02X%02X%02X", address, new_address, type, baudrate,
	  moreconfig);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * read_from_channel - Read analog input from channel N
 */

int read_from_channel (int fd, int address, int channel)
{
  char cmd[5];
  int snd;

  sprintf(cmd, "#%02X%1X", address, channel);
  
  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * span_calibration - Calibrates an analog input module to correct for
 *                    gain errors.
 */

int span_calibration (int fd, int address)
{
  char cmd[4];
  int snd;

  sprintf(cmd, "$%02X0", address);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * offset_calibration - Calibrates an analog input module to correct for
 *                      offset errors.
 */

int offset_calibration (int fd, int address)
{
  char cmd[4];
  int snd;
  
  sprintf(cmd, "#%02X1", address);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * config_status - Requests the return of the configuration data from
 *                 from the analog input module at address passed as
 *                 argument.
 */

int config_status (int fd, int address)
{
  char cmd[4];
  int snd;

  sprintf(cmd, "$%02X2", address);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * int en_dis_channels - Enable/Disable channels for multiplexing.
 */

int en_dis_channels (int fd, int address, int channels_to_manage)
{
  char cmd[6];
  int snd;

  sprintf (cmd, "$%02X5%02X", address, channels_to_manage);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * read_channel_status - Asks a specified input module to return the status
 *                       of all eight channels.
 */

int read_channel_status (int fd, int address)
{
  char cmd[4];
  int snd;

  sprintf (cmd, "$%02X6", address);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * sync_sample - Orders all analog input modules to sample their input values
 *               and store the values in special registers.
 */

int sync_sample (int fd)
{
  char cmd[3];
  int snd;

  sprintf (cmd, "#**");

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * read_sync_sample - Returns the input value that was stored in the addressed
 *                    module's register, after Synchronized Sampling command
 *                    was issued.
 */

int read_sync_sample (int fd, int address)
{
  char cmd[4];
  int snd;

  sprintf (cmd, "$%02X4", address);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * read_version - Read Firmware version.
 */

int read_version (int fd, int address)
{
  char cmd[4];
  int snd;

  sprintf (cmd, "$%02XF", address);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * read_module_name - Read module name
 */

int read_module_name (int fd, int address)
{
  char cmd[4];
  int snd;

  sprintf (cmd, "$%02XM", address);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * CJC_status - Instructs the addressed analog input module to read its
 *              Cold Junction Compensation (CJC) sensors and return the 
 *              acquired data.
 */

int CJC_status (int fd, int address)
{
  char cmd[4];
  int snd;

  sprintf (cmd, "$%02X3", address);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * CJC_offset_calibration - Calibrates ana analog input module to adjust for
 *                          offset errors of its CJC sensors.
 */

int CJC_offset_calibration (int fd, int address, char sign)
{
  char cmd[5];
  int snd;

  sprintf (cmd, "$%02X9%c", address, sign);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * set_mem_config - Sets the channel storage status, standalone mode, data
 *                  logger mode and sampling interval for the specified analog
 *                  input data logger.
 */

int set_mem_config (int fd, int address, int channels, int standalone_mode,
                    int data_log_mode, int sample_interval)
{
  char cmd[12];
  int snd;

  sprintf (cmd, "@%02XC%2d%01d%01d%4d", address, channels, standalone_mode,
	   data_log_mode, sample_interval);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * read_mem_config - Requests the configuration data from the analog input
 *                   data logger at adress AA.
 */

int read_mem_config (int fd, int address)
{
  char cmd[4];
  int snd;

  sprintf (cmd, "@%02XD", address);

  snd = sendcmd(fd, cmd);
  
  return snd;
}

/*
 * set_mem_operation_mode - Sets the operation mode of the analog input data
 *                          logger at adress AA to Start or Stop.
 */

int set_mem_operation_mode (int fd, int address, int mode)
{
  char cmd[5];
  int snd;

  sprintf (cmd, "@%02dS%01d", address, mode);

 snd = sendcmd(fd,cmd);

 return snd;
}

/*
 * read_mem_opertaion_mode - Request the memory operation status of the analog
 *                           input data logger at adress AA.
 */

int read_mem_opertaion_mode (int fd, int address)
{
 char cmd[4];
 int snd;
 
 sprintf (cmd, "@%02XT", address);
 
 snd = sendcmd(fd,cmd);

 return snd;
}

/*
 * event_rec_count - Request the number of event records stored in the analog
 *                   input data logger at adress AA.
 */

int event_rec_count (int fd, int address)
{
 char cmd[4];
 int snd;
 
 sprintf (cmd, "@%02XL", address);
 
 snd = sendcmd(fd,cmd);

 return snd;
}

/*
 * standard_rec_count - Request the number of standard records stored in the
 *                      analog input data logger at adress AA.
 */

int standard_rec_count (int fd, int address)
{
 char cmd[4];
 int snd;
 
 sprintf (cmd, "@%02XN", address);
 
 snd = sendcmd(fd,cmd);

 return snd;
}

/*
 * read_rec_content - Request the content of record NNNN stored in the analog
 *                    input data logger at adress AA.
 */

int read_rec_content (int fd, int address, int stored_record_index)
{
  char cmd[8];
  int snd;

  sprintf (cmd, "@%02XR%04d", address, stored_record_index);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * set_alarm_limit - Set high/low alarm limits for the channel C in the analog
 *                   input data logger at adress AA.
 */

int set_alarm_limit (int fd, int address, int channel, int sign_high, 
		     int decimal_high, int high_limit, int sign_low,
		     int decimal_low, int low_limit)
{
  char cmd[17];
  int snd;

  sprintf (cmd, "@%02XA%01d%01d%01d%04d%01d%01d%04d", address, channel,
	   sign_high, decimal_high, high_limit, sign_low, decimal_low,
	   low_limit);

  snd = sendcmd(fd, cmd);

  return snd;
}

/*
 * read_alarm_limit - Request alarm limits for the specified channel in the
 *                    abalog input data logger at adress AA.
 */

int read_alarm_limit (int fd, int address, int channel)
{
  char cmd[5];
  int snd;

  sprintf (cmd, "@%02XB%01d", address, channel);

  snd = sendcmd(fd, cmd);

  return snd;
}
